#include <iostream>
#include <iomanip>
#include <fstream>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string>
#include <float.h>
#include <limits.h>
#include "rof-dhm-event-ver310.h"
#include "cell-dhm-event-ver320.h"

#ifdef _OPENMP
#include <omp.h>        // OpenMP
#endif

using namespace std;

// 1k-dhm version 321 ̉
// 20200704 10Ԋu̗ʂo͂悤ɂByt
// 20190511 R搶쐬1bn`f[^ɑΉBn`f[^̕\bPʂŗ^悤ɂByt

//֐̐錾difinition of the function
int makeBasinList(int row, int col, class Cell** object, int*& numOfSameBasin, int& numOfCalcCell, int**& orderedCellNum);
void read2DValue(ROF** fROF, int Ndays, int ROFnum_row, int ROFnum_col);
void read2DValue4Binary(ROF** fROF, int Ndays, int ROFnum_row, int ROFnum_col);

int main()
{
	cout << "Reading parameters for runoff simulation." << endl;
	ifstream finParam("./param-dhm-event-ver310.dat");
	if(!finParam){
		cout << "./param-dhm-event-ver310.dat cannot open." << endl;
		exit(1);
	}

	// vZ̈ simulation range
	cout << "1: Reading parameters for study area." << endl;

	float wTopoScale;

	finParam >> wTopoScale;
	cout << "  Topography scale : " << wTopoScale << endl;

	int wCol,wRow;
	double wSWLON, wSWLAT;

	finParam >> wCol; 
	finParam >> wRow; 
	cout << "  Number of col : " << wCol << "  Number of row : " << wRow << endl;

	finParam >> wSWLON; 
	finParam >> wSWLAT;
	cout << " Sorth West corner Longitude (degree) : " << wSWLON 
		 << " Latitude (deg) : " << wSWLAT << endl;

	const float TopoScale = wTopoScale;	// DEM ̕\bPʂŐݒ肷 set DEM resolution with second
	const int Col = wCol;			// Ώ̗ۗ̈ column number of study area
	const int Row = wRow;			// Ώۗ̈̍s row number of study area
	const double SWLON = wSWLON;		// Ώۗ̈̓쐼[ox South west corner of longitude (degree)
	const double SWLAT = wSWLAT;		// Ώۗ̈̓쐼[ܓx South west corner of latiitude (degree)

	// ͗L~Jf[^̈ range of input effective rainfall data
	cout << "2: Reading parameters for input data." << endl;

	int wROFdata_type;	// added by Tanaka T. 2018/6/28
	finParam >> wROFdata_type;
	cout << "Read ";
	if(wROFdata_type == 1){
		cout << "./input/rain.dat" << endl;
	}
	else if(wROFdata_type == 2){
		cout << "./input/rain.bin" << endl;
	}
	else{
		cout << "Set rainfall data format (1: text, 2: 4 byte binary)" << endl;
		exit(1);
	}
	const int ROFdata_type = wROFdata_type;

	int wROFnum_col, wROFnum_row;
	double wDelta_x, wDelta_y, wROF_WLON, wROF_SLAT;
	finParam >> wROFnum_col;
	finParam >> wROFnum_row;
	cout << "  Number of col : " << wROFnum_col 
		 << "  Number of row : " << wROFnum_row << endl;

	finParam >> wDelta_x;
	finParam >> wDelta_y;
	cout << "  Input data grid size of x-direction : " << wDelta_x << endl;
	cout << "  Input data grid size of y-direction : " << wDelta_y << endl;

	finParam >> wROF_WLON; 
	cout << "  Longitude of the center of the Southwest corner of input data (degree) : " << wROF_WLON << endl;
	finParam >> wROF_SLAT; 
	cout << "  Latitude of the center of the Southwest corner of input data (degree) : " << wROF_SLAT << endl;
													
	const int ROFnum_col = wROFnum_col;	// column number of input data
	const int ROFnum_row = wROFnum_row;	//s row number of input data
	const double Delta_x = wDelta_x;	//oxiq longitudinal resolution
	const double Delta_y = wDelta_y;	//ܓxiq latitudinal resolution
	const double ROF_WLON = wROF_WLON;	//쐼[ox South west corner of longitude (degree)
	const double ROF_SLAT = wROF_SLAT;	//쐼[ܓx South west corner of latitude (degree)

	// vZ simulation term
	cout << "3: Reading parameters for simuration duration." << endl;
	int wNdays;
	finParam >> wNdays;
	cout << "  Simulation duration (days) : " << wNdays << endl;

	const int Ndays = wNdays; // vZ Simulation duration (days)

	//  finite difference condition
	cout << "4: Reading parameters for finite difference condition." << endl;
	int wTime_Step, wNspace; double wTheta;
	finParam >> wTime_Step; cout << "  Finite difference time step (sec) : " << wTime_Step << endl;
	finParam >> wNspace; cout << "  Finite difference space step : " << wNspace << endl;
	finParam >> wTheta; cout << "  Finite difference weight : " << wTheta << endl;

	if(600%wTime_Step != 0) {
		cout << endl << "    Error : Time step is wrong!" << endl << endl;
		exit(1);
	}

	const int Time_Step = wTime_Step; // vZ̎ԊԊu [sec] similation time step [sec] */
	const int Nspace = wNspace; /* vZ̋ԕ number of spatial divisions */
	const double Theta = wTheta; 	/* d݌W (-) 0.5 < theta <= 1.0 */
	const int Ntime = 3600/Time_Step;  // ̓f[^Ԋu(1)ł̌vZ

	// model parameter setting for runoff simulation vZ̃p^
	cout << "5: Reading parameters for kinematic wave model." << endl;
	double wMan_channel, wMan_slope, wKa, wDa, wDm, wBeta;

	finParam >> wMan_slope; cout << "  Manning roughmess coefficient for slope [m-s unit] : " << wMan_slope << endl;
	finParam >> wKa; cout << "  Hydraulic conductivity [m-s] : " << wKa << endl;
	finParam >> wDa; cout << "  Depth of capillary and non-capillary soil layers [m] : " << wDa << endl;
	finParam >> wDm; cout << "  Depth of capillary soil layers [m] : " << wDm << endl;
	finParam >> wBeta; cout << "  Exponent constant of unsaturated hydraulic conductivity : " << wBeta << endl;
	finParam >> wMan_channel; cout << "  Manning roughmess coefficient for channel [m-s unit] : " << wMan_channel << endl;
	if(wMan_slope <= 0.0){
		cout << endl << "    Error : Manning roughmess coefficient for slope must be larger than zero" << endl << endl;
		exit(1);
	}
	if(wKa < 0.0){
		cout << endl << "    Error : Hydraulic conductivity must be larger than zero" << endl << endl;
		exit(1);
	}
	if(wDa < 0.0){
		cout << endl << "    Error : Depth of capillary and non-capillary soil layers must be larger than zero" << endl << endl;
		exit(1);
	}
	if(wDm < 0.0){
		cout << endl << "    Error : Depth of capillary soil layers must be larger than zero" << endl << endl;
		exit(1);
	}
	if(wBeta < 0.0){
		cout << endl << "    Error : Exponent constant of unsaturated hydraulic conductivity must be larger than zero" << endl << endl;
		exit(1);
	}
	if(wMan_channel <= 0.0){
		cout << endl << "    Error : Manning roughmess coefficient for channel must be larger than zero" << endl << endl;
		exit(1);
	}
	if(wDa < wDm){
		cout << endl << "    Error : Depth of capillary soil layer must be larger than that of non-capillary soil layer" << endl << endl;
		exit(1);
	}
	
	// topographic parameters n`p[^
	cout << "6: Reading the parameters of topographic conditions." << endl;
	double wsL, wsB; int wThreshold_basin;
	finParam >> wThreshold_basin; cout << "  Area to classify slope and channel : " << wThreshold_basin << endl;
	const int Threshold_basin = wThreshold_basin; // WObh臒libasin > threshold_basin ݂Obh͓ɂȂj

	finParam >> wsL; cout << "  Slope length [m] : " << wsL << endl; // the parameter of slope length
	//finParam >> wsB; cout << "  Slope width [m] : " << wsB << endl; // the parameter of slope width

	if(wsL <= 0.0){
		cout << endl << "    Error : Slope length must be larger than zero" << endl << endl;
		exit(1);
	}

	//if(wsB <= 0.0){
	//	cout << endl << "    Error : Slope width must be larger than zero" << endl << endl;
	//	exit(1);
	//}

	/*  initial condition */
	cout << "7: Reading itinial discharge for kinematic wave model." << endl;
	double winitialROF;
	finParam >> winitialROF; cout << "  Initial runoff height [mm/hr] : " << winitialROF << endl;
	const double intialROF = winitialROF; // initial runoff height [mm/hr]

	if(winitialROF <= 0.0){
		cout << endl << "    Error : Initial runoff height must be larger than zero" << endl << endl;
		exit(1);
	}

	finParam.close();
	//--------------------- end parameter reading ------------------------------------------------


	/*--- ϐ̒` dif-variables ---*/
	int iday, ihour, dt;
	int n, k;
	bool executeOrPass = false;
	//errno_t error;
	
	FILE *fCorrectedMed;
	float med;

	FILE *fDown;
	FILE *fBasin;
	FILE *fRiverNumber;

	float ROFlat;
	float ROFlon;
	char DischargeFileName[256];
	char ConditionFileName[256];
	int* upperGridNum;
	int* numOfSameBasin;
	int** orderedCellNum;
	int numOfBasin;
	int numOfCalcCell;

	// reading watershed numbers for simulation from riverNumList.dat vZ闬̔ԍ̐ݒ
	ifstream fin1("./topoData/riverNumList.dat");
	if(!fin1){
		cout << "riverNumList.dat cannot open.\n";
		return 1;
	}

	int size_riverNumber_cal=0;
	string ss;

	while (getline(fin1, ss)) { // count number of rivers.  ΏۂƂ͐쐔𒲂ׂ
		size_riverNumber_cal++; 
	}
	fin1.close();
	cout << "Number of rivers for simulation : " << size_riverNumber_cal << endl << endl;

	int* riverNumber_cal;
	riverNumber_cal = new int[size_riverNumber_cal];

	ifstream fin2("./topoData/riverNumList.dat");
	if(!fin2){
		cout << "riverNumList.txt cannot open.\n";
		return 1;
	}
	int iriver, icol, irow, ilat, ilon, iarea;
	double dlon, dlat;
	for(int i=0; i<size_riverNumber_cal; i++){ // setting river number for simulation
		fin2 >> iriver >> iarea >> icol >> irow >> ilon >> ilat >> dlon >> dlat;
		riverNumber_cal[i]=iriver;
	}
	fin2.close();



	/*--- vZZ̃m Allocate simulation cell ---*/
	// Memory assignment for grid cell of simulation
	class Cell** cellObject; 
	cellObject = new class Cell*[Row];
	for(int i=0; i<Row; i++){
		cellObject[i] = new class Cell[Col];
	}


	/*--- ̓f[^Z̃m Allocate memories of input data ---*/
	class ROF** rofObject;
	rofObject = new class ROF*[ROFnum_row];
	for(int i=0; i<ROFnum_row; i++){
		rofObject[i] = new class ROF[ROFnum_col];
	}



	/*--- ̓f[^IuWFNgɏ Setting of input data information ---*/
	
	for(int i=0; i<ROFnum_row; i++){
		ROFlat = ROF_SLAT + i*Delta_y ;  // set latitude. Runoff grid is the center.
		for(int j=0; j<ROFnum_col; j++){
			ROFlon = ROF_WLON +  j*Delta_x; // set longitude  Runoff grid is the center.
			rofObject[i][j].setInformation(&ROFlat, &ROFlon, j, i, Ndays);
		}
	}


	/*--- ͓ԁEoϐɒl set the sream informations ---*/

	/* latitude  longitude */
	if ((fCorrectedMed = fopen("./topoData/modDem.bin", "rb")) == NULL) {
		cout << "modDem.bin cannot open." << endl;
		exit(1);
	}

	for(int i=Row-1; i>=0; i--) { // DEM data arranges from south to north
		for(int j=0; j<Col; j++) {
			fread(&med, sizeof(float), 1, fCorrectedMed);
			cellObject[i][j].setElevation(i, j, Row, Col, SWLAT, SWLON, med, TopoScale);
			cellObject[i][j].setNULL();
			//cout << "row=" << i << " col=" << j << endl;
		}
	}
	fclose(fCorrectedMed);

	/* down  basin downStreamGrid riverNumber */
	if ((fDown = fopen("./topoData/flowDir.bin", "rb")) == NULL) {
		cout << "flowDir.bin cannot open." << endl;
		exit(1);
	}

	if ((fBasin = fopen("./topoData/flowAcc.bin", "rb")) == NULL) {
		cout << "flowAcc.bin cannot open." << endl;
		exit(1);
	}

	if ((fRiverNumber = fopen("./topoData/riverNum.bin", "rb")) == NULL) {
		cout << "riverNum.bin cannot open." << endl;
		exit(1);
	}

	for(int i=Row-1; i>=0; i--) { // Topo data arranges from south to north
		for(int j=0; j<Col; j++) {

			fread(&cellObject[i][j].down, sizeof(int), 1, fDown);
			fread(&cellObject[i][j].basin, sizeof(int), 1, fBasin);
			fread(&cellObject[i][j].riverNumber, sizeof(int), 1, fRiverNumber);

			if(cellObject[i][j].riverNumber < 0) {
				continue;
			}
			cellObject[i][j].setDownStreamGrid(i, j, cellObject);
		}
	}

	fclose(fDown);
	fclose(fBasin);
	fclose(fRiverNumber);

	/* upStreamGrid riverNumber */
	// 㗬Obh̏ݒ肷B
	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++)	{
			// ΏۃObhCAΏۗ̈̒[A[̏ꍇ͗`ĂȂ̂Ŕ΂B
			// Ώۗ̈̒[Ƃ͈ԊÕObhłB
			// 
			if(cellObject[i][j].riverNumber < 0 || cellObject[i][j].isThisGridEnd==true || cellObject[i][j].down==0) {
				continue;
			}
			//cout << i << ", " << j << endl;
			cellObject[i][j].setUpStreamGrid();
		}
	}

	/* numberOfUpStreamGrid */
	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++){
			if(cellObject[i][j].riverNumber < 0 || cellObject[i][j].isThisGridEnd==true) {
				continue;
			}
			cellObject[i][j].countNumberOfUpStreamGrid();
		}
	}


	/*--- ROFCell ԗʂ p^ set parameters ---*/
	
	// originally edited by matsubara 2013
	// Revised by Tanaka T. 10/18/2015
	int ROFnum_col_data = -9999;
	int ROFnum_row_data = -9999;
	double Delta_x_data = -9999.0;
	double Delta_y_data = -9999.0;
	double ROF_WLON_data = -9999.0;
	double ROF_SLAT_data = -9999.0;;
	sprintf(ConditionFileName, "./temp/setROFCell.dat");
	ifstream fsrc(ConditionFileName);
	if(fsrc){
		
		fsrc >> ROFnum_col_data;
		fsrc >> ROFnum_row_data;
		fsrc >> Delta_x_data;
		fsrc >> Delta_y_data;
		fsrc >> ROF_WLON_data;
		fsrc >> ROF_SLAT_data;
	}
	fsrc.close(); 

	//p[^t@CƓ̓f[^t@C̍Wl̍
	double resDx = fabs(Delta_x-Delta_x_data);
	double resDy = fabs(Delta_y-Delta_y_data);
	double resLon = fabs(ROF_WLON_data-ROF_WLON);
	double resLat = fabs(ROF_SLAT_data-ROF_SLAT);
	if(
		ROFnum_col_data != ROFnum_col ||
		ROFnum_row_data != ROFnum_row ||
		resDx > DBL_MIN ||
		resDy > DBL_MIN ||
		resLon> DBL_MIN ||
		resLat> DBL_MIN )
	{
	//setROFCell.dat݂Ȃ܂para-event2.datƐȂꍇ͐Vɍ 2015/10/18 Tanaka T.
	//vZ@ɍ킹āAǂݍ񂾃f[^̏_ȉ̌ɍ킹덷ݒ肷 2015/10/18 Tanaka T.
		//cout << "cannot open " << ConditionFileName << " or " 
		//	<< ConditionFileName << " and param-event-ver2.dat does not match" << endl;
		cout << "Make ROF data " << ConditionFileName << endl;

		ofstream fsrc(ConditionFileName); 
		if(!fsrc){
			cout << ConditionFileName << " cannot open.\n";
			//exit(1);
		}
		
		fsrc <<	ROFnum_col << endl;
		fsrc <<	ROFnum_row << endl;
		fsrc <<	setprecision(15) << Delta_x << endl;
		fsrc <<	setprecision(15) << Delta_y << endl;
		fsrc <<	setprecision(15) << ROF_WLON << endl;
		fsrc <<	setprecision(15) << ROF_SLAT << endl;
		
		int ix, iy;
		for(int i=0; i<Row; i++){
			for(int j=0; j<Col; j++){
				int *rofRC= new int[2];//0: Row, 1: Column
				cellObject[i][j].setROFCell(rofObject, ROFnum_row, ROFnum_col, rofRC);
				fsrc << i << " " << j << " " << rofRC[0] << " " << rofRC[1] << endl;
			}
		}
		fsrc.close();
	}
	else{ // t@C݂ꍇ́Ałɍ쐬ꂽt@Cǂ
		cout << "Read ROF data from " << ConditionFileName << endl;
		
		ifstream fsrc(ConditionFileName);

		fsrc >> ROFnum_col_data;
		fsrc >> ROFnum_row_data;
		fsrc >> Delta_x_data;
		fsrc >> Delta_y_data;
		fsrc >> ROF_WLON_data;
		fsrc >> ROF_SLAT_data;

		int ii, jj, ix, iy;
		for(int i=0; i<Row; i++){
			for(int j=0; j<Col; j++){
				fsrc >> ii >> jj >> ix >> iy ;
				cellObject[i][j].PutROFCell(rofObject, ix, iy);
			}
		}
		fsrc.close();
	}
	

	// ԗʂ̏ƃp^̐ݒ set parameters and initial conditions
	cout << "Setting parameter data." << endl;
	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++){
			
			if(cellObject[i][j].riverNumber < 0 || cellObject[i][j].isThisGridEnd == true) {
				continue;
			}

			cellObject[i][j].Set_parameter(wMan_channel, wMan_slope, wKa, wDa, wDm, wBeta, wsL, wsB, Threshold_basin, Nspace, Time_Step, Theta, Ntime, Ndays);
			cellObject[i][j].initialize(intialROF);

			cellObject[i][j].gridKind = 1;  // ΖʃObh
			if (cellObject[i][j].basin > Threshold_basin) {
				cellObject[i][j].gridKind = 2;  // ͐Obh
			}
		}
	}

	// ͌̃p^̐ݒ set parameters at the river mouthes

	cout << "Setting parameter data at the river mouthes." << endl;
	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++){

			if(cellObject[i][j].down == 0) {
				cellObject[i][j].Set_parameterRiverMouth();
				//cout << "Gradient at river mouthes " << cellObject[i][j].mG << endl;
			}
		}
	}


	/*--- ʌvZ̐ arrange calculation order ---*/
	int Col_data = -9999;
	int Row_data = -9999;
	double SWLON_data = -9999.0;
	double SWLAT_data = -9999.0;

	sprintf(ConditionFileName, "./temp/numOfBasin.dat");
	ifstream finNB(ConditionFileName);
	int *rnumOfSameBasin;

	//t@C݂ꍇ́A܂t@C̒n`ǂݎ
	if(finNB){
		finNB >> Col_data;
		finNB >> Row_data;
		finNB >> SWLON_data;
		finNB >> SWLAT_data;
	}
	finNB.close();

	//p[^t@CƓ̓f[^t@C̍Wl̍
	double resLon2 = fabs(SWLON_data - SWLON);
	double resLat2 = fabs(SWLAT_data - SWLAT);

	//t@C̑ݗLђn`̐mF
	if(Col_data !=	Col ||
	   Row_data !=	Row ||
	   resLon2 > DBL_MIN ||
	   resLat2 > DBL_MIN )
	{//numOfBasin.dat݂Ȃ܂para-event2.datƐȂꍇ͐Vɍ 2015/10/18 Tanaka T.
	//vZ@ɍ킹āAǂݍ񂾃f[^̏_ȉ̌ɍ킹덷ݒ肷 2015/10/18 Tanaka T.
	//cout << "cannot open " << ConditionFileName << " or " 
	//	<< ConditionFileName << " and param-event-ver2.dat does not match" << endl;
		cout << "Make basinList data " << ConditionFileName << endl;

		numOfBasin = makeBasinList(Row, Col, cellObject, numOfSameBasin, numOfCalcCell, orderedCellNum);

		ofstream foutNB(ConditionFileName);
		if(!foutNB){
			cout << ConditionFileName << " cannot open.\n";
			//exit(1);
		}
		
		foutNB << Col << endl;
		foutNB << Row << endl;
		foutNB << setprecision(15) << SWLON << endl;
		foutNB << setprecision(15) << SWLAT << endl;
		
		foutNB << numOfBasin << endl ;
		foutNB << numOfCalcCell << endl ;

		for(int i=0; i<numOfBasin; i++){
			foutNB << numOfSameBasin[i] << endl ;
		}
		foutNB.close();

		cout << "Make calculation order data ./temp/orderedCell.dat" << endl;
		ofstream foutOrderedCell("./temp/orderedCell.dat");
		if(!foutOrderedCell){
			cout << "./temp/orderedCell.dat cannot open.\n";
			//exit(1);
		}
		
		for(int i=0; i<numOfCalcCell; i++){
			foutOrderedCell << cellObject[orderedCellNum[i][0]][orderedCellNum[i][1]].basin << " " 
				<< orderedCellNum[i][0] << " " 
				<< orderedCellNum[i][1] << endl;
		}
		foutOrderedCell.close();

	}
	else{  // t@C݂ꍇ́Ałɍ쐬ꂽt@Cǂ
		ifstream finNB(ConditionFileName);
		cout << "Read basinList data from " << ConditionFileName << endl;

		finNB >> Col_data;
		finNB >> Row_data;
		finNB >> SWLON_data;
		finNB >> SWLAT_data;
		finNB >> numOfBasin;
		finNB >> numOfCalcCell;
		
		numOfSameBasin = new int[numOfBasin];
		for(int i=0; i<numOfBasin; i++){
			finNB >> numOfSameBasin[i];
		}
		finNB.close();

		ifstream finOrderedCell("./temp/orderedCell.dat");
		if(!finOrderedCell){
			cout << "./temp/orderedCell.dat cannot open.\n";
			exit(1);
		}
		cout << "Read calculation order data from ./temp/orderedCell.dat" << endl << endl;
		
		orderedCellNum = new int*[numOfCalcCell];
		for(int i=0; i<numOfCalcCell; i++){
			orderedCellNum[i] = new int[2];
		}
		int ii;
		for(int i=0; i<numOfCalcCell; i++){
			finOrderedCell >> ii >> orderedCellNum[i][0] >> orderedCellNum[i][1];
		}
		finOrderedCell.close();
	}

	cout << "*** Number of different kind of cell accumulattion number : " << numOfBasin << endl;
	cout << "Number of calculation cell number : " << numOfCalcCell << endl;
	cout << "Total simulation time is " << Ndays << " days." << endl << endl; 
	
	
	/*---̓f[^̐ݒ setting of input data ---*/
	// Read 2-dimensional input data
	if(ROFdata_type == 1){
		read2DValue(rofObject, Ndays, ROFnum_row, ROFnum_col);
	}
	else{
		read2DValue4Binary(rofObject, Ndays, ROFnum_row, ROFnum_col);
	}

	/*---_̐ݒ setting of dam ---*/
	// Editied by Miyawaki K. 2015
	/* 
	[definition]
		flood : ^ߊJn[m^3/s]
		storagePL : \ʂ̒[m^3]
		storageNL : ^ʂ̒(e)[m^3]
		storageSL : T[`[Wʂ̒(L)[m^3]
	
	[ʂpꍇ]
		damSimulationKind0B
		inputreal[_].datpӂB
	*/
	
	int wDamSimulationKind; // 0: use observed dam outflow, 1: use dam model
	int wNumOfDam, wDamID, wDamCol, wDamRow;
	double wFlood, wStoragePL, wStorageNL, wStorageSL;
	string sDamName;
	char cDamName[20];

	//ϐ̐錾Ɉړ Tanaka T. 2015/1/22
	int numOfDam=0;
	class damCell* damObject;

	cout << '\n' << "Read ./input/dam.dat" << '\n';
	cout << "-------------Dam setting-------------\n";
	ifstream finDam("./input/dam.dat");
	if(!finDam){
		cout << "dam.dat does not exist. It is assumed no dams exist." << endl;
	}
	else{
		finDam >> wNumOfDam;
			numOfDam = wNumOfDam;
			damObject = new class damCell[numOfDam];
		
		for(int id=0; id<numOfDam; id++){
			finDam >> wDamID >> wDamSimulationKind >> wDamCol >> wDamRow >> wFlood >> wStoragePL >> wStorageNL >> wStorageSL >> sDamName;
			strcpy(cDamName, sDamName.c_str());
			damObject[id].setDamGrid(cDamName, wDamID, wDamCol, wDamRow, Col, Row, cellObject, wDamSimulationKind);
			damObject[id].setDamInitialState(Ndays, Ntime);
			damObject[id].setDamParameter(wFlood, wStoragePL, wStorageNL, wStorageSL);
		}
		finDam.close();
	}
	cout << "-------------------------------------\n";

	/*--- r̐ݒ setting of reservoir ---*/
	// Editied by Miyawaki K. 2015
		
	int wNumOfRes, wResID, wResCol, wResRow, wrK;
	string sResName;
	char cResName[20];
	
	cout << '\n' << "Read ./input/reservoir.dat" << '\n';
	cout << "----------Reservoir setting----------\n";
	ifstream finRes("./input/reservoir.dat");

	//ϐ̐錾Ɉړ Tanaka T. 2015/1/22
	int numOfRes=0;
	class resCell* resObject;

	if(!finRes){
		cout << "reservoir.dat dose not exist. It is assumed no reservoirs exist." << endl;
	}
	else{
		finRes >> wNumOfRes;
		numOfRes = wNumOfRes;
		resObject = new class resCell[numOfRes];
		
		for(int id=0; id<numOfRes; id++){
			finRes >> wResID >> wResCol >> wResRow >> wrK >> sResName;
			strcpy(cResName, sResName.c_str());
			resObject[id].setResGrid(cResName, wResID, wResCol, wResRow, Col, Row, cellObject);
			resObject[id].setResInitialState();
			resObject[id].setResParameter(wrK);
		}
		
		finRes.close();
	}
	cout << "-------------------------------------\n";

	
	/*--- ovZ runoff simulation ---*/
	// Simulation order sytem was changed. 2014/11/30 Tanaka
	cout << endl; 
	cout << "Runoff simulation starts." << endl; 

	int startBasin, endBasin;
	for(int ib = 0; ib < numOfBasin; ib++){
		//vZiqԍ̐ݒ
		if(ib == 0){ startBasin = 0;}
		else{ startBasin = endBasin + 1;}
		endBasin = startBasin + numOfSameBasin[ib] - 1;
		
		cout << "  " <<  setprecision(3) << startBasin+1<<"/"<<numOfCalcCell << " ends.\n";
		//#pragma omp parallel for private(iday, ihour, dt, executeOrPass, k) if(numOfSameBasin[ib]>2*omp_get_num_procs())
		#pragma omp parallel for private(iday, ihour, dt, executeOrPass, k)
		for(int i=startBasin; i<=endBasin; i++){ // the same basin group
			if(cellObject[orderedCellNum[i][0]][orderedCellNum[i][1]].gridKind < 0) { // edge of study area 
				continue;
			}
			
			// w肵ɑObĥ݁Aȉ̑s 
			// Only catchments specified in 'riverNumberList.txt' is calculated.
			executeOrPass = false;

			for(k=0; k<size_riverNumber_cal; k++){
				if(cellObject[orderedCellNum[i][0]][orderedCellNum[i][1]].riverNumber == riverNumber_cal[k]){
					executeOrPass = true;
					break;
				}
			}

			if(executeOrPass == false) {
				continue;
			}

			switch(cellObject[orderedCellNum[i][0]][orderedCellNum[i][1]].gridKind){
			case 2:  // ͓Obh
				for(iday=1; iday <= Ndays; iday++){ // day loop
					for(ihour=1; ihour<=24; ihour++){ // hour loop
						//cout << "͐Obh ib = " << ib << " ihour = " << ihour << endl;
						cellObject[orderedCellNum[i][0]][orderedCellNum[i][1]].getLateralRunoff(iday, ihour);
						for(dt=0; dt < Ntime; dt++){ // simulation time step roop 
							cellObject[orderedCellNum[i][0]][orderedCellNum[i][1]].CalculateRiver(iday, ihour, dt);
						}
					}
				}
				break;
			case 1:  // ΖʃObh
				for(iday=1; iday <= Ndays; iday++){ // day loop
					for(ihour=1; ihour<=24; ihour++){ // hour loop
						//cout << "ΖʃObh ib = " << ib << " ihour = " << ihour << endl;
						cellObject[orderedCellNum[i][0]][orderedCellNum[i][1]].getLateralRunoff(iday, ihour);
						for(dt=0; dt<Ntime; dt++){ // simulation time step roop 
							cellObject[orderedCellNum[i][0]][orderedCellNum[i][1]].CalculateBasin(iday, ihour, dt);
						}
					}
				}
				break;
			case 99: //_Obh
				for(int id=0; id<numOfDam; id++){
					if(damObject[id].damRow == orderedCellNum[i][0] && damObject[id].damCol == orderedCellNum[i][1]){
						for(iday=1; iday <= Ndays; iday++){ // day loop
							for(ihour=1; ihour<=24; ihour++){ // hour loop
								cellObject[orderedCellNum[i][0]][orderedCellNum[i][1]].getLateralRunoff(iday, ihour);
								for(dt=0; dt<Ntime; dt++){ // simulation time step roop 
									damObject[id].calculateDamOutflow(iday, ihour, dt);
									//cout << "ihour = " << ihour << " dt = " << dt << endl;
								}
							}
						}
					}
				}
				break;
			case 100: //`rObh
				for(int id=0; id<numOfRes; id++){
					if(resObject[id].resRow == orderedCellNum[i][0] && resObject[id].resCol == orderedCellNum[i][1]){
						for(iday=1; iday <= Ndays; iday++){ // day loop
							for(ihour=1; ihour<=24; ihour++){ // hour loop
								cellObject[orderedCellNum[i][0]][orderedCellNum[i][1]].getLateralRunoff(iday, ihour);
								for(dt=0; dt<Ntime; dt++){ // simulation time step roop 
									resObject[id].calculateResOutflow(iday, ihour, dt);
									//cout << "ihour = " << ihour << " dt = " << dt << endl;
								}
							}
						}
					}
				}
				break;
			}
		}
	}


	cout << endl; 
	cout << "Runoff simulation ends." << endl; 



	/*--- vZʂ̏o Output simulation results ---*/
	// o̓t@C̏ Setting of output files
	
	ofstream File10minSnapShot; // 10 minutes snapshot flow
	//sprintf_s(DischargeFileName, 256, "./output/dischargeHourlyMaxInDay.bin");
	sprintf(DischargeFileName, "./output/discharge10minSnapshot.bin");
	cout << "Write " << DischargeFileName << endl;
	File10minSnapShot.open(DischargeFileName, ios::out | ios::binary);

	ofstream File10minMean; // 10 minutes mean flow
	//sprintf_s(DischargeFileName, 256, "./output/dischargeDaily.bin");
	sprintf(DischargeFileName, "./output/discharge10minMean.bin");
	cout << "Write " << DischargeFileName << endl;
	File10minMean.open(DischargeFileName, ios::out | ios::binary);
	
	ofstream FileHourlyMean; // Hourly mean flow
	//sprintf_s(DischargeFileName, 256, "./output/dischargeHourly.bin");
	sprintf(DischargeFileName, "./output/dischargeHourlyMean.bin");
	cout << "Write " << DischargeFileName << endl;
	FileHourlyMean.open(DischargeFileName, ios::out | ios::binary);

	//ofstream FileDaily; // Daily mean flow
	////sprintf_s(DischargeFileName, 256, "./output/dischargeDaily.bin");
	//sprintf(DischargeFileName, "./output/dischargeDaily.bin");
	//cout << "Write " << DischargeFileName << endl;
	//FileDaily.open(DischargeFileName, ios::out | ios::binary);
	
	//ofstream FileHourlyMax; // Maximum hourly flow in a day
	////sprintf_s(DischargeFileName, 256, "./output/dischargeHourlyMaxInDay.bin");
	//sprintf(DischargeFileName, "./output/dischargeHourlyMaxInDay.bin");
	//cout << "Write " << DischargeFileName << endl;
	//FileHourlyMax.open(DischargeFileName, ios::out | ios::binary);

	//write sptial total rainfall to a file
	ofstream FileTotalRain; // Point total rainfall data
	sprintf(DischargeFileName, "./output/totalRainDistribution.asc");
	cout << "Write " << DischargeFileName << endl;
	FileTotalRain.open(DischargeFileName);
	double **totalRain = new double*[Row];
	for(int m=0; m<Row; m++){
		totalRain[m] = new double[Col];
	}
	for(int m=0; m<Row; m++){
		for(int n=0; n<Col; n++){
			totalRain[m][n] = 0.0;
		}
	}

	/*--- for debugging --------------------------------------------*/
	//ofstream FileTest; // Point hourly data
	//sprintf_s(DischargeFileName, 256, "./output/dischargeTest.dat");
	////sprintf(DischargeFileName, "./output/dischargeTest.dat");
	//cout << "Write " << DischargeFileName << endl;
	//FileTest.open(DischargeFileName);
	//sprintf_s(DischargeFileName, 256, "./output/totalRainDistribution.dat");
	/*--- for debugging--------------------------------------------- */

	// ʂ̏o Output
	float Nodata= -9999;
	const int Ntime10 = 600/Time_Step; // o̓f[^Ԋu(10)ł̌vZ

	float tenMinSnapShotFlow; // m3/hr
	float tenMinMeanFlow; // m3/hr
	float hourlyMeanFlow; // m3/hr
	//float dailyFlow; // m3/hr
	//float hourlyFlowMax; // m3/hr

	for(iday=0; iday < Ndays; iday++) {

	// ---daily flow output---
	//	for(int m=Row-1; m >=0; m--){ // latitude direction from South for Grads format
	//		for(n=0; n<Col; n++){     // longitude direction from West
	//			if(cellObject[m][n].isThisGridEnd==true){
	//				FileDaily.write((char*)&Nodata, sizeof(float));
	//				FileHourlyMax.write((char*)&Nodata, sizeof(float));
	//				continue;
	//			}
	//			// w肵ɑȂObhɂ false ݒ肷B 
	//			// 'false' is set to the grids of outside study area.
	//			executeOrPass = false;
	//			for(k=0; k<size_riverNumber_cal; k++){
	//				if(cellObject[m][n].riverNumber == riverNumber_cal[k]){
	//					executeOrPass = true;
	//					break;
	//				}
	//			}
	//			if(executeOrPass == false){ // ΏۊÕObhł Nodata Ń[vi߂B
	//				FileDaily.write((char*)&Nodata, sizeof(float));
	//				FileHourlyMax.write((char*)&Nodata, sizeof(float));
	//				continue;
	//			}
	//				
	//			dailyFlow = (float) cellObject[m][n].GetDailyFlow(iday, Ntime);
	//			FileDaily.write((char*)&dailyFlow, sizeof(float));
	//
	//			hourlyFlowMax = (float) cellObject[m][n].GetHourlyFlowMax(iday, Ntime);
	//			FileHourlyMax.write((char*)&hourlyFlowMax, sizeof(float));
	//		}
	//	}

		//	---hourly flow output---
		for (ihour=0; ihour < 24; ihour++) {
			for(int m=Row-1; m >=0; m--){ // latitude direction from South for Grads format
				for(n=0; n<Col; n++){     // longitude direction from West
					totalRain[m][n] += cellObject[m][n].ROFCell->value[iday+1][ihour];
					
					if(cellObject[m][n].isThisGridEnd==true){
						FileHourlyMean.write((char*)&Nodata, sizeof(float));
						continue;
					}
					// w肵ɑȂObhɂ false ݒ肷B 
					// 'false' is set to the grids of outside study area.
					executeOrPass = false;
					for(k=0; k<size_riverNumber_cal; k++){
						if(cellObject[m][n].riverNumber == riverNumber_cal[k]){
							executeOrPass = true;
							break;
						}
					}
					if(executeOrPass == false){ // ΏۊÕObhł Nodata Ń[vi߂B
						FileHourlyMean.write((char*)&Nodata, sizeof(float));
						continue;
					}

					hourlyMeanFlow = (float) cellObject[m][n].GetHourlyFlow(iday, ihour, Ntime);
					FileHourlyMean.write((char*)&hourlyMeanFlow, sizeof(float));

				}
			}

			//	---10 minutes flow output---
			for (dt=0; dt < 6; dt++) { // begin 10min output loop
				for(int m=Row-1; m >=0; m--){ // latitude direction from South for Grads format
					for(n=0; n<Col; n++){     // longitude direction from West

						if(cellObject[m][n].isThisGridEnd==true){
							File10minSnapShot.write((char*)&Nodata, sizeof(float));
							File10minMean.write((char*)&Nodata, sizeof(float));
							continue;
						}
						// w肵ɑȂObhɂ false ݒ肷B 
						// 'false' is set to the grids of outside study area.
						executeOrPass = false;
						for(k=0; k<size_riverNumber_cal; k++){
							if(cellObject[m][n].riverNumber == riverNumber_cal[k]){
								executeOrPass = true;
								break;
							}
						}
						if(executeOrPass == false){ // ΏۊÕObhł Nodata Ń[vi߂B
							File10minSnapShot.write((char*)&Nodata, sizeof(float));
							File10minMean.write((char*)&Nodata, sizeof(float));
							continue;
						}
					
						tenMinSnapShotFlow = (float) cellObject[m][n].Get10minSnapShotFlow(iday, ihour, dt, Ntime10);
						File10minSnapShot.write((char*)&tenMinSnapShotFlow, sizeof(float));

						tenMinMeanFlow = (float) cellObject[m][n].Get10minMeanFlow(iday, ihour, dt, Ntime10);
						File10minMean.write((char*)&tenMinMeanFlow, sizeof(float));
					}
				}
			} // end 10min output loop
		} // end hourly output loop

		/*--- File output for debugging by YT -------------------------------------------------*/
		//int outCol = 59; // begin from 1 from east
		//int outRow = 22; // begin from 1 from south
		//outRow = Row -outRow + 1; // change from north
		//for (ihour=0; ihour < 24; ihour++) {
		//	hourlyFlow = (float) cellObject[outRow-1][outCol-1].GetHourlyFlow(iday, ihour, Ntime);
		//	FileTest << iday+1 << " " << ihour << " " << hourlyFlow << endl;
		//}
		/*---debugging--------------------------------------------------------------------------*/

	} // end daily output loop

	File10minSnapShot.close();
	File10minMean.close();
	FileHourlyMean.close();
	//FileDaily.close();
	//FileHourlyMax.close();
	/*--- debugging ------------------*/
	//FileTest.close();
	/*---------------------------------*/

	
	// ovZɓ͂~Jf[^̑~Jʕzo͂
	// Output spatial distribution of total rainfall, edited by Tanak T. 10/18/2015
	// Write in the ASCII format
	/*----------------------------------------------------------------------------------------------*/
	FileTotalRain << "ncols "<<Col<<"\n";
	FileTotalRain << "nrows "<<Row<<"\n";
	FileTotalRain << "xllcorner " << SWLON <<"\n";
	FileTotalRain << "yllcorner " << SWLAT <<"\n";
	FileTotalRain << "cellsize " <<0.00833333 << "\n";
	FileTotalRain << "NODATA_value "<< Nodata << "\n";
	for(int m=0; m<Row; m++){
		for(int n=0; n<Col; n++){
			FileTotalRain << totalRain[m][n] << " ";
		}
		FileTotalRain << endl;
	}
	FileTotalRain.close();


	// _ʁEʁEoʂo͂
	// Output dam/pondage control, editied by Miyawaki K. 2015
	/*----------------------------------------------------------------------------------------------*/
	cout << "Write ./output/damData/damIQS.dat\n";
	for(int id=0; id<numOfDam; id++){
		int count = 0;
		float hourlyI, hourlyQ, hourlyS;
		string fileIQS;
		fileIQS = damObject[id].damName;
		fileIQS = "./output/damData/" + fileIQS + "IQS.dat";
		ofstream foutDam(fileIQS.c_str());
		
		for(iday=0; iday<Ndays; iday++){
			for(ihour=0; ihour<24; ihour++){
				hourlyI = damObject[id].damGrid->GetHourlyInflow(iday, ihour, Ntime);
				hourlyQ = damObject[id].damGrid->GetHourlyFlow(iday, ihour, Ntime);
				hourlyS = damObject[id].GetHourlyStorage(iday, ihour, Ntime);
				count++;
				foutDam << count << '\t' << hourlyI << '\t' << hourlyQ << '\t' << hourlyS << '\n';
			}
		}
		foutDam.close();
	}

	// r̗ʁEoʂo͂
	/*----------------------------------------------------------------------------------------------*/
	cout << "Write ./output/resData/resIQ.dat\n";
	for(int id=0; id<numOfRes; id++){
		int count = 0;
		float hourlyI, hourlyQ;
		string fileIQ;
		fileIQ = resObject[id].resName;
		fileIQ = "./output/resData/" + fileIQ + "IQ.dat";
		ofstream foutRes(fileIQ.c_str());
		
		for(iday=0; iday<Ndays; iday++){
			for(ihour=0; ihour<24; ihour++){
				hourlyI = resObject[id].resGrid->GetHourlyInflow(iday, ihour, Ntime);
				hourlyQ = resObject[id].resGrid->GetHourlyFlow(iday, ihour, Ntime);
				count++;
				foutRes << count << '\t' << hourlyI << '\t' << hourlyQ << '\n';
			}
		}
		foutRes.close();
	}




/*--- vZ̊J Delete memories used in simulation ---*/
	// vZZ
	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++){
			if(cellObject[i][j].isThisGridEnd==true || cellObject[i][j].riverNumber < 0){
				continue;
			}
			cellObject[i][j].deleteQandA(Ndays);
		}
	}

	// ̓f[^
	for(int i=0; i<Row; i++){
		delete[] cellObject[i];
	}
	delete[] cellObject;

	for(int i=0; i<ROFnum_row; i++){
		delete[] rofObject[i];
	}
	delete[] rofObject;


	// _Erp
	for(int i=0; i<numOfDam; i++){
		damObject[i].deleteS(Ndays);
	}
	if(numOfDam>0){
		delete[] damObject;
	}
	if(numOfRes>0){
		delete[] resObject;
	}


	return 0;
}


/*--- ֐̒` ---*/

//ovZŽvZݒ肷
//vZ̕ύX 2014/11/30 Tanaka T.
int makeBasinList(int row, int col, class Cell** object, int*& numOfSameBasin, int& numOfCalcCell, int**& orderedCellNum)
{
	//dĂȂWZ
	int num = 0;

	int** basin;

	basin = new int*[row];
	for(int i=0; i<row; i++){
		
		basin[i] = new int[col];
	}


	//eIuWFNgbasinzbasinɂ
	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			basin[i][j] = object[i][j].basin;
		}
	}

	//ɕבւ
	int tmp;

	//̂ƂɁCבւi,jԖڂ̃ẐƂƂ̃ZԍLĂ 2014/11/28 Tanaka
	int ***org = new int**[row];
	int itmp;
	for(int i=0; i<row; i++){
		org[i] = new int*[col];
	}
	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			org[i][j] = new int[2];
		}
	}
	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
//			org[i][j][0] = -9999;
//			org[i][j][1] = -9999;
			org[i][j][0] = i;
			org[i][j][1] = j;
		}
	}


	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			
			for(int jj=j+1; jj<col; jj++){

				if(basin[i][j] > basin[i][jj]){

					tmp = basin[i][j];
					basin[i][j] = basin[i][jj];
					basin[i][jj] = tmp;

					//ƂƂ̃ZԍL
					itmp = org[i][j][0];
					org[i][j][0] = org[i][jj][0];
					org[i][jj][0] = itmp;
					itmp = org[i][j][1];
					org[i][j][1] = org[i][jj][1];
					org[i][jj][1] = itmp;
				}
			}
			
			for(int ii=i+1; ii<row; ii++){
				for(int jj=0; jj<col; jj++){

					if(basin[i][j] > basin[ii][jj]){

						tmp = basin[i][j];
						basin[i][j] = basin[ii][jj];
						basin[ii][jj] = tmp;

						//ƂƂ̃ZԍL
						itmp = org[i][j][0];
						org[i][j][0] = org[ii][jj][0];
						org[ii][jj][0] = itmp;
						itmp = org[i][j][1];
						org[i][j][1] = org[ii][jj][1];
						org[ii][jj][1] = itmp;
					}
				}
			}

		}
	}


	//zbasin̗vfɏdĂȂ͉邩
	int dammy = 0;
	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			
			if(basin[i][j] > 0 && basin[i][j] != dammy){
				dammy = basin[i][j];
				num++;
			}
		}
	}

	//z upperGridNum ̃m
	int *upperGridNum = new int[num];

	//z upperGridNum basin̒lɓ
	//̎_dammybasin̍őlDbasin[0][0]Ƃ͕KقȂ邩dammybasin[0][0]ɒuD
	int k=0;
	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			if(basin[i][j] > 0){
				if(basin[i][j] != dammy){
					*(upperGridNum+k) = basin[i][j];
					dammy = basin[i][j];
					k++;
				}
			}
		}
	}

	//basin̒lŽ𐔂 2014/11/28 Tanaka
	//܂C̈ԏbasin̒ldammyɓ
	numOfSameBasin = new int[num];
	for(int i=0; i<num; i++){
		numOfSameBasin[i]=0;
	}
	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			if(basin[i][j] > 0 && basin[i][j] < dammy){
				dammy = basin[i][j];
			}
		}
	}
	k=0;
	numOfCalcCell = 0;
	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			if(basin[i][j] > 0){
				numOfCalcCell++;
				
				if(basin[i][j] != dammy){
					dammy = basin[i][j];
					k++;
					numOfSameBasin[k]++;
				}
				else{
					numOfSameBasin[k]++;
				}
			}
		}
	}

	//z orderedCellNum basin̒l̏ɕבւZ̔ԍ 2014/11/28 Tanaka
	orderedCellNum = new int*[numOfCalcCell];
	for(int i=0; i<numOfCalcCell; i++){
		orderedCellNum[i] = new int[2];
	}
	for(int i=0; i<numOfCalcCell; i++){
		for(int j=0; j<2; j++){
			orderedCellNum[i][j] = -9999;
		}
	}
	int sameBi=0;
	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			if(basin[i][j] > 0){
				orderedCellNum[sameBi][0] = org[i][j][0];
				orderedCellNum[sameBi][1] = org[i][j][1];
				sameBi++;
			}
		}
	}

	for(int i=0; i<row; i++){
		delete[] basin[i];
	}
	delete[] basin;
	/*------------------------------------------------------------------*/

	return num;
}


// ̓f[^ASCII`œǂݍ
// read input data file arranged for two dimensional data in ASCII format. 
void read2DValue(ROF** fROF, int Ndays, int ROFnum_row, int ROFnum_col)
{

	char ROFfilename[256];

	//sprintf_s(ROFfilename, 256, "./input/rain.dat");
	sprintf(ROFfilename, "./input/rain.dat");
	cout << "Read " << ROFfilename << endl;

	ifstream fin(ROFfilename);
	if(!fin){
		cout << "ROF file cannot open.\n";
		exit(1);
	}

	int year, month, day, hour, xcol, yrow;
	for(int iday=1; iday <= Ndays; iday++){
		for(int ihour=0; ihour<24; ihour++){
			fin >> year >> month >> day >> hour >> xcol >> yrow;
			
			// fROF arrays from south
			for(int i=ROFnum_row-1; i>=0; i--){
				for(int j=0; j<ROFnum_col; j++){
					
					fin >> fROF[i][j].value[iday][ihour] ;
				}
			}
		}
	}

	fin.close();
}

// ̓f[^4 byte binary`œǂݍ
// read input data file arranged for two dimensional data in binary format. 
void read2DValue4Binary(ROF** fROF, int Ndays, int ROFnum_row, int ROFnum_col)
{
	char ROFfilename[256];

	sprintf(ROFfilename, "./input/rain.bin");
	cout << "Read " << ROFfilename << endl;

	ifstream fin(ROFfilename, ios::in | ios::binary);
	if(!fin){
		cout << "ROF file cannot open.\n";
		exit(1);
	}

	float *rain = new float [ROFnum_row*ROFnum_col];
	int year, month, day, hour, xcol, yrow;
	for(int iday=1; iday <= Ndays; iday++){
		for(int ihour=0; ihour<24; ihour++){
			
			fin.read((char*)rain, sizeof(float)*ROFnum_row*ROFnum_col);
			
			// fROF arrays from south
			for(int i=0; i<ROFnum_row; i++){
				for(int j=0; j<ROFnum_col; j++){
					
					fROF[i][j].value[iday][ihour] = rain[i*ROFnum_col + j];
				}
			}
		}
	}

	fin.close();
}

